/*
 * To change this template, choose Tools | Templates and open the template in
 * the editor.
 */
package org.netbeans.modules.php.drupal.ui.customizer;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import javax.swing.JOptionPane;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import org.netbeans.api.project.Project;
import org.netbeans.modules.php.drupal.util.Util;
import org.netbeans.modules.php.drupal.util.LibraryParser;
import org.netbeans.modules.php.drupal.DrupalDevelPreferences;
import org.netbeans.spi.project.ui.support.ProjectCustomizer.Category;

/**
 *
 * @author Jamie Holly <jamie@hollyit.net>
 */
public class DrupalCustomizerPanel extends javax.swing.JPanel {

    private final Category category;
    private final Project phpModule;

    DrupalCustomizerPanel(Category category, Project phpModule) {
        assert category != null;
        assert phpModule != null;

        this.category = category;
        this.phpModule = phpModule;

        this.category.setStoreListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                storeData();
            }
        });

        initComponents();


        drupalLibraryPathTextField.setText(DrupalDevelPreferences.getLibraryPath(phpModule));

        txtDrupalFolder.setText(DrupalDevelPreferences.getDrupalPath(phpModule));
        if (getLibraryPath().equals(DrupalDevelPreferences.getDefaultLibraryPath())) {
            drupalLibraryPathTextField.setText("");
        }
        drupalLibraryPathTextField.getDocument().addDocumentListener(new DocumentListener() {

            @Override
            public void insertUpdate(DocumentEvent e) {
                validateLibraryPath();
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                validateLibraryPath();
            }

            @Override
            public void changedUpdate(DocumentEvent e) {
                validateLibraryPath();
            }
        });

        validateLibraryPath();
        // validateData();
    }

    private void validateLibraryPath() {
        updateDrupalVersions();
    }

    private void updateDrupalVersions() {
        this.cbDrupalVersion.removeAllItems();
        String drVer = DrupalDevelPreferences.getDrupalVersion(phpModule);
        ArrayList items = new ArrayList();
        if (drVer.equals("") || drVer.equals("Default")) {
            drVer = DrupalDevelPreferences.getDefaultDrupalVersion();
        }

        String path = DrupalDevelPreferences.libraryInstallPath() + "/code";
        items = LibraryParser.parseTree(path);
        cbDrupalVersion.addItem("Default");
        for (int i = 0; i < items.size(); i++) {
            String item = items.get(i).toString();
            Object obj = Util.makeObj(item);
            cbDrupalVersion.addItem(obj);
            if (item.toString().equals(drVer)) {
                cbDrupalVersion.setSelectedItem(obj);
            }
        }

    }

    private void mergeList(ArrayList list, String item) {
        Boolean addit = true;
        for (int i = 0; i < list.size(); i++) {
            if (list.get(i).equals(item)) {
                addit = false;
            }
        }

        if (addit) {
            list.add(item);
        }


    }

    void storeData() {
        if (getLibraryPath().equals(DrupalDevelPreferences.getDefaultLibraryPath())) {
            drupalLibraryPathTextField.setText("");
        }
        DrupalDevelPreferences.setDrupalVersion(phpModule, getDrupalVersion());

        DrupalDevelPreferences.setLibraryPath(phpModule, getLibraryPath());
        DrupalDevelPreferences.setDrupalPath(phpModule, getDrupalPath());

    }

    private String getDrupalPath() {
        return txtDrupalFolder.getText().toString();
    }

    private String getLibraryPath() {
        return drupalLibraryPathTextField.getText().toString();
    }

    private String getDrupalVersion() {
        return cbDrupalVersion.getSelectedItem().toString();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        cbDrupalVersion = new javax.swing.JComboBox();
        jPanel1 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        drupalLibraryPathTextField = new javax.swing.JTextField();
        jButton1 = new javax.swing.JButton();
        txtDrupalFolder = new javax.swing.JTextField();
        jButton3 = new javax.swing.JButton();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();

        jLabel1.setText(org.openide.util.NbBundle.getMessage(DrupalCustomizerPanel.class, "DrupalCustomizerPanel.jLabel1.text")); // NOI18N

        cbDrupalVersion.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));

        jLabel2.setText(org.openide.util.NbBundle.getMessage(DrupalCustomizerPanel.class, "DrupalCustomizerPanel.jLabel2.text")); // NOI18N

        drupalLibraryPathTextField.setText(org.openide.util.NbBundle.getMessage(DrupalCustomizerPanel.class, "DrupalCustomizerPanel.drupalLibraryPathTextField.text")); // NOI18N

        jButton1.setText(org.openide.util.NbBundle.getMessage(DrupalCustomizerPanel.class, "DrupalCustomizerPanel.jButton1.text")); // NOI18N
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel2)
                        .addContainerGap(341, Short.MAX_VALUE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(drupalLibraryPathTextField)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(jButton1)
                        .addGap(22, 22, 22))))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(drupalLibraryPathTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButton1))
                .addContainerGap())
        );

        txtDrupalFolder.setText(org.openide.util.NbBundle.getMessage(DrupalCustomizerPanel.class, "DrupalCustomizerPanel.txtDrupalFolder.text")); // NOI18N

        jButton3.setText(org.openide.util.NbBundle.getMessage(DrupalCustomizerPanel.class, "DrupalCustomizerPanel.jButton3.text")); // NOI18N
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        jLabel3.setText(org.openide.util.NbBundle.getMessage(DrupalCustomizerPanel.class, "DrupalCustomizerPanel.jLabel3.text")); // NOI18N

        jLabel4.setFont(new java.awt.Font("Tahoma", 2, 11));
        jLabel4.setText(org.openide.util.NbBundle.getMessage(DrupalCustomizerPanel.class, "DrupalCustomizerPanel.jLabel4.text")); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(jLabel1)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(cbDrupalVersion, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addComponent(jLabel3))
                        .addContainerGap())
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(jLabel4, javax.swing.GroupLayout.DEFAULT_SIZE, 430, Short.MAX_VALUE)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(txtDrupalFolder)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jButton3)))
                        .addGap(22, 22, 22))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(6, 6, 6)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(cbDrupalVersion, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel3)
                .addGap(2, 2, 2)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(txtDrupalFolder, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButton3))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel4)
                .addContainerGap(144, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        Util.browseDrupalLibraryPath(this, drupalLibraryPathTextField);
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        Util.browseDrupalLibraryPath(this, txtDrupalFolder);
    }//GEN-LAST:event_jButton3ActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox cbDrupalVersion;
    private javax.swing.JTextField drupalLibraryPathTextField;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton3;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JTextField txtDrupalFolder;
    // End of variables declaration//GEN-END:variables
}
